"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import UnrecognizedStr
from typing import Literal, Union


TypesIntegrationEntityType = Union[
    Literal[
        "customer",
        "plan",
        "invoice",
        "subscription",
        "payment",
        "credit_note",
        "addon",
        "item",
        "item_price",
        "price",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel
from flexprice_py.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class DeleteEntitlementRequestTypedDict(TypedDict):
    id: str
    r"""Entitlement ID"""


class DeleteEntitlementRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Entitlement ID"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .types_aggregationtype import TypesAggregationType
from .types_windowsize import TypesWindowSize
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class MeterAggregationTypedDict(TypedDict):
    bucket_size: NotRequired[TypesWindowSize]
    field: NotRequired[str]
    r"""Field is the key in $event.properties on which the aggregation is to be applied
    For ex if the aggregation type is sum for API usage, the field could be \"duration_ms\" 
    """
    group_by: NotRequired[str]
    r"""GroupBy is the property name in event.properties to group by before aggregating.
    Currently only supported for MAX aggregation with bucket_size.
    When set, aggregation is applied per unique value of this property within each bucket,
    then the per-group results are summed to produce the bucket total.
    """
    multiplier: NotRequired[str]
    r"""Multiplier is the multiplier for the aggregation
    For ex if the aggregation type is sum_with_multiplier for API usage, the multiplier could be 1000
    to scale up by a factor of 1000. If not provided, it will be null.
    """
    type: NotRequired[TypesAggregationType]


class MeterAggregation(BaseModel):
    bucket_size: Optional[TypesWindowSize] = None

    field: Optional[str] = None
    r"""Field is the key in $event.properties on which the aggregation is to be applied
    For ex if the aggregation type is sum for API usage, the field could be \"duration_ms\" 
    """

    group_by: Optional[str] = None
    r"""GroupBy is the property name in event.properties to group by before aggregating.
    Currently only supported for MAX aggregation with bucket_size.
    When set, aggregation is applied per unique value of this property within each bucket,
    then the per-group results are summed to produce the bucket total.
    """

    multiplier: Optional[str] = None
    r"""Multiplier is the multiplier for the aggregation
    For ex if the aggregation type is sum_with_multiplier for API usage, the multiplier could be 1000
    to scale up by a factor of 1000. If not provided, it will be null.
    """

    type: Optional[TypesAggregationType] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["bucket_size", "field", "group_by", "multiplier", "type"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .types_billingcadence import TypesBillingCadence
from .types_billingperiod import TypesBillingPeriod
from .types_filtercondition import TypesFilterCondition, TypesFilterConditionTypedDict
from .types_sortcondition import TypesSortCondition, TypesSortConditionTypedDict
from .types_status import TypesStatus
from .types_subscriptionstatus import TypesSubscriptionStatus
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


TypesSubscriptionFilterOrder = Literal[
    "asc",
    "desc",
]


class TypesSubscriptionFilterTypedDict(TypedDict):
    active_at: NotRequired[str]
    r"""ActiveAt filters subscriptions that are active at the given time"""
    billing_cadence: NotRequired[List[TypesBillingCadence]]
    r"""BillingCadence filters by billing cadence"""
    billing_period: NotRequired[List[TypesBillingPeriod]]
    r"""BillingPeriod filters by billing period"""
    customer_id: NotRequired[str]
    r"""CustomerID filters by customer ID"""
    end_time: NotRequired[str]
    expand: NotRequired[str]
    external_customer_id: NotRequired[str]
    r"""ExternalCustomerID filters by external customer ID"""
    filters: NotRequired[List[TypesFilterConditionTypedDict]]
    invoicing_customer_ids: NotRequired[List[str]]
    r"""InvoicingCustomerIDs filters by invoicing customer ID"""
    limit: NotRequired[int]
    offset: NotRequired[int]
    order: NotRequired[TypesSubscriptionFilterOrder]
    parent_subscription_ids: NotRequired[List[str]]
    r"""ParentSubscriptionIDs filters by parent subscription IDs"""
    plan_id: NotRequired[str]
    r"""PlanID filters by plan ID"""
    sort: NotRequired[List[TypesSortConditionTypedDict]]
    start_time: NotRequired[str]
    status: NotRequired[TypesStatus]
    subscription_ids: NotRequired[List[str]]
    subscription_status: NotRequired[List[TypesSubscriptionStatus]]
    r"""SubscriptionStatus filters by subscription status"""
    with_line_items: NotRequired[bool]
    r"""WithLineItems includes line items in the response"""


class TypesSubscriptionFilter(BaseModel):
    active_at: Optional[str] = None
    r"""ActiveAt filters subscriptions that are active at the given time"""

    billing_cadence: Optional[List[TypesBillingCadence]] = None
    r"""BillingCadence filters by billing cadence"""

    billing_period: Optional[List[TypesBillingPeriod]] = None
    r"""BillingPeriod filters by billing period"""

    customer_id: Optional[str] = None
    r"""CustomerID filters by customer ID"""

    end_time: Optional[str] = None

    expand: Optional[str] = None

    external_customer_id: Optional[str] = None
    r"""ExternalCustomerID filters by external customer ID"""

    filters: Optional[List[TypesFilterCondition]] = None

    invoicing_customer_ids: Optional[List[str]] = None
    r"""InvoicingCustomerIDs filters by invoicing customer ID"""

    limit: Optional[int] = None

    offset: Optional[int] = None

    order: Optional[TypesSubscriptionFilterOrder] = None

    parent_subscription_ids: Optional[List[str]] = None
    r"""ParentSubscriptionIDs filters by parent subscription IDs"""

    plan_id: Optional[str] = None
    r"""PlanID filters by plan ID"""

    sort: Optional[List[TypesSortCondition]] = None

    start_time: Optional[str] = None

    status: Optional[TypesStatus] = None

    subscription_ids: Optional[List[str]] = None

    subscription_status: Optional[List[TypesSubscriptionStatus]] = None
    r"""SubscriptionStatus filters by subscription status"""

    with_line_items: Optional[bool] = None
    r"""WithLineItems includes line items in the response"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "active_at",
                "billing_cadence",
                "billing_period",
                "customer_id",
                "end_time",
                "expand",
                "external_customer_id",
                "filters",
                "invoicing_customer_ids",
                "limit",
                "offset",
                "order",
                "parent_subscription_ids",
                "plan_id",
                "sort",
                "start_time",
                "status",
                "subscription_ids",
                "subscription_status",
                "with_line_items",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .types_alertsettings import TypesAlertSettings, TypesAlertSettingsTypedDict
from .types_alertstate import TypesAlertState
from .types_autotopup import TypesAutoTopup, TypesAutoTopupTypedDict
from .types_creditbreakdown import TypesCreditBreakdown, TypesCreditBreakdownTypedDict
from .types_status import TypesStatus
from .types_walletconfig import TypesWalletConfig, TypesWalletConfigTypedDict
from .types_walletstatus import TypesWalletStatus
from .types_wallettype import TypesWalletType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoWalletResponseTypedDict(TypedDict):
    alert_settings: NotRequired[TypesAlertSettingsTypedDict]
    alert_state: NotRequired[TypesAlertState]
    auto_topup: NotRequired[TypesAutoTopupTypedDict]
    balance: NotRequired[str]
    config: NotRequired[TypesWalletConfigTypedDict]
    conversion_rate: NotRequired[str]
    r"""amount in the currency =  number of credits * conversion_rate
    ex if conversion_rate is 1, then 1 USD = 1 credit
    ex if conversion_rate is 2, then 1 USD = 0.5 credits
    ex if conversion_rate is 0.5, then 1 USD = 2 credits
    """
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    credit_balance: NotRequired[str]
    credits_available_breakdown: NotRequired[TypesCreditBreakdownTypedDict]
    currency: NotRequired[str]
    customer_id: NotRequired[str]
    description: NotRequired[str]
    environment_id: NotRequired[str]
    id: NotRequired[str]
    metadata: NotRequired[Dict[str, str]]
    name: NotRequired[str]
    status: NotRequired[TypesStatus]
    tenant_id: NotRequired[str]
    topup_conversion_rate: NotRequired[str]
    r"""topup_conversion_rate is the conversion rate for the topup to the currency
    ex if topup_conversion_rate is 1, then 1 USD = 1 credit
    ex if topup_conversion_rate is 2, then 1 USD = 0.5 credits
    ex if topup_conversion_rate is 0.5, then 1 USD = 2 credits
    """
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]
    wallet_status: NotRequired[TypesWalletStatus]
    wallet_type: NotRequired[TypesWalletType]


class DtoWalletResponse(BaseModel):
    alert_settings: Optional[TypesAlertSettings] = None

    alert_state: Optional[TypesAlertState] = None

    auto_topup: Optional[TypesAutoTopup] = None

    balance: Optional[str] = None

    config: Optional[TypesWalletConfig] = None

    conversion_rate: Optional[str] = None
    r"""amount in the currency =  number of credits * conversion_rate
    ex if conversion_rate is 1, then 1 USD = 1 credit
    ex if conversion_rate is 2, then 1 USD = 0.5 credits
    ex if conversion_rate is 0.5, then 1 USD = 2 credits
    """

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    credit_balance: Optional[str] = None

    credits_available_breakdown: Optional[TypesCreditBreakdown] = None

    currency: Optional[str] = None

    customer_id: Optional[str] = None

    description: Optional[str] = None

    environment_id: Optional[str] = None

    id: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    name: Optional[str] = None

    status: Optional[TypesStatus] = None

    tenant_id: Optional[str] = None

    topup_conversion_rate: Optional[str] = None
    r"""topup_conversion_rate is the conversion rate for the topup to the currency
    ex if topup_conversion_rate is 1, then 1 USD = 1 credit
    ex if topup_conversion_rate is 2, then 1 USD = 0.5 credits
    ex if topup_conversion_rate is 0.5, then 1 USD = 2 credits
    """

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    wallet_status: Optional[TypesWalletStatus] = None

    wallet_type: Optional[TypesWalletType] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "alert_settings",
                "alert_state",
                "auto_topup",
                "balance",
                "config",
                "conversion_rate",
                "created_at",
                "created_by",
                "credit_balance",
                "credits_available_breakdown",
                "currency",
                "customer_id",
                "description",
                "environment_id",
                "id",
                "metadata",
                "name",
                "status",
                "tenant_id",
                "topup_conversion_rate",
                "updated_at",
                "updated_by",
                "wallet_status",
                "wallet_type",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_address import DtoAddress, DtoAddressTypedDict
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoTenantBillingDetailsTypedDict(TypedDict):
    address: NotRequired[DtoAddressTypedDict]
    email: NotRequired[str]
    help_email: NotRequired[str]
    phone: NotRequired[str]


class DtoTenantBillingDetails(BaseModel):
    address: Optional[DtoAddress] = None

    email: Optional[str] = None

    help_email: Optional[str] = None

    phone: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["address", "email", "help_email", "phone"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .types_cancelimmediatelyinvoicepolicy import TypesCancelImmediatelyInvoicePolicy
from .types_cancellationtype import TypesCancellationType
from .types_prorationbehavior import TypesProrationBehavior
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoCancelSubscriptionRequestTypedDict(TypedDict):
    cancellation_type: TypesCancellationType
    cancel_immediately_inovice_policy: NotRequired[TypesCancelImmediatelyInvoicePolicy]
    proration_behavior: NotRequired[TypesProrationBehavior]
    reason: NotRequired[str]
    r"""Reason for cancellation (for audit and business intelligence)"""


class DtoCancelSubscriptionRequest(BaseModel):
    cancellation_type: TypesCancellationType

    cancel_immediately_inovice_policy: Optional[TypesCancelImmediatelyInvoicePolicy] = (
        None
    )

    proration_behavior: Optional[TypesProrationBehavior] = None

    reason: Optional[str] = None
    r"""Reason for cancellation (for audit and business intelligence)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["cancel_immediately_inovice_policy", "proration_behavior", "reason"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .types_filtercondition import TypesFilterCondition, TypesFilterConditionTypedDict
from .types_sortcondition import TypesSortCondition, TypesSortConditionTypedDict
from .types_status import TypesStatus
from .types_transactionreason import TypesTransactionReason
from .types_transactionstatus import TypesTransactionStatus
from .types_transactiontype import TypesTransactionType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


TypesWalletTransactionFilterOrder = Literal[
    "asc",
    "desc",
]


class TypesWalletTransactionFilterTypedDict(TypedDict):
    created_by: NotRequired[str]
    credits_available_gt: NotRequired[float]
    end_time: NotRequired[str]
    expand: NotRequired[str]
    expiry_date_after: NotRequired[str]
    expiry_date_before: NotRequired[str]
    filters: NotRequired[List[TypesFilterConditionTypedDict]]
    r"""filters allows complex filtering based on multiple fields"""
    id: NotRequired[str]
    limit: NotRequired[int]
    offset: NotRequired[int]
    order: NotRequired[TypesWalletTransactionFilterOrder]
    priority: NotRequired[int]
    reference_id: NotRequired[str]
    reference_type: NotRequired[str]
    sort: NotRequired[List[TypesSortConditionTypedDict]]
    start_time: NotRequired[str]
    status: NotRequired[TypesStatus]
    transaction_reason: NotRequired[TypesTransactionReason]
    transaction_status: NotRequired[TypesTransactionStatus]
    type: NotRequired[TypesTransactionType]


class TypesWalletTransactionFilter(BaseModel):
    created_by: Optional[str] = None

    credits_available_gt: Optional[float] = None

    end_time: Optional[str] = None

    expand: Optional[str] = None

    expiry_date_after: Optional[str] = None

    expiry_date_before: Optional[str] = None

    filters: Optional[List[TypesFilterCondition]] = None
    r"""filters allows complex filtering based on multiple fields"""

    id: Optional[str] = None

    limit: Optional[int] = None

    offset: Optional[int] = None

    order: Optional[TypesWalletTransactionFilterOrder] = None

    priority: Optional[int] = None

    reference_id: Optional[str] = None

    reference_type: Optional[str] = None

    sort: Optional[List[TypesSortCondition]] = None

    start_time: Optional[str] = None

    status: Optional[TypesStatus] = None

    transaction_reason: Optional[TypesTransactionReason] = None

    transaction_status: Optional[TypesTransactionStatus] = None

    type: Optional[TypesTransactionType] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "created_by",
                "credits_available_gt",
                "end_time",
                "expand",
                "expiry_date_after",
                "expiry_date_before",
                "filters",
                "id",
                "limit",
                "offset",
                "order",
                "priority",
                "reference_id",
                "reference_type",
                "sort",
                "start_time",
                "status",
                "transaction_reason",
                "transaction_status",
                "type",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .types_entitytype import TypesEntityType
from .types_filetype import TypesFileType
from .types_tasktype import TypesTaskType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreateTaskRequestTypedDict(TypedDict):
    entity_type: TypesEntityType
    file_type: TypesFileType
    file_url: str
    task_type: TypesTaskType
    file_name: NotRequired[str]
    metadata: NotRequired[Dict[str, Any]]


class DtoCreateTaskRequest(BaseModel):
    entity_type: TypesEntityType

    file_type: TypesFileType

    file_url: str

    task_type: TypesTaskType

    file_name: Optional[str] = None

    metadata: Optional[Dict[str, Any]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["file_name", "metadata"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from flexprice_py import errors, models, utils
from flexprice_py._hooks import HookContext
from flexprice_py.types import OptionalNullable, UNSET
from flexprice_py.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, Mapping, Optional


class Integrations(BaseSDK):
    def get_integration(
        self,
        *,
        provider: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoSecretResponse:
        r"""Get integration details

        Use when you need to check or display integration config (e.g. which provider is linked). Sensitive values may be redacted.

        :param provider: Integration provider
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetIntegrationRequest(
            provider=provider,
        )

        req = self._build_request(
            method="GET",
            path="/secrets/integrations/by-provider/{provider}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getIntegration",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoSecretResponse, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_integration_async(
        self,
        *,
        provider: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoSecretResponse:
        r"""Get integration details

        Use when you need to check or display integration config (e.g. which provider is linked). Sensitive values may be redacted.

        :param provider: Integration provider
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetIntegrationRequest(
            provider=provider,
        )

        req = self._build_request_async(
            method="GET",
            path="/secrets/integrations/by-provider/{provider}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getIntegration",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoSecretResponse, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def create_or_update_integration(
        self,
        *,
        provider_param: str,
        credentials: Dict[str, str],
        name: str,
        provider: models.TypesSecretProvider,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoSecretResponse:
        r"""Create or update an integration

        Use when storing or updating credentials for an external integration (e.g. Stripe, HubSpot). Secrets are encrypted at rest.

        :param provider_param: Integration provider
        :param credentials:
        :param name:
        :param provider:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateOrUpdateIntegrationRequest(
            provider_param=provider_param,
            body=models.DtoCreateIntegrationRequest(
                credentials=credentials,
                name=name,
                provider=provider,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/secrets/integrations/create/{provider}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body, False, False, "json", models.DtoCreateIntegrationRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createOrUpdateIntegration",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.DtoSecretResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def create_or_update_integration_async(
        self,
        *,
        provider_param: str,
        credentials: Dict[str, str],
        name: str,
        provider: models.TypesSecretProvider,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoSecretResponse:
        r"""Create or update an integration

        Use when storing or updating credentials for an external integration (e.g. Stripe, HubSpot). Secrets are encrypted at rest.

        :param provider_param: Integration provider
        :param credentials:
        :param name:
        :param provider:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateOrUpdateIntegrationRequest(
            provider_param=provider_param,
            body=models.DtoCreateIntegrationRequest(
                credentials=credentials,
                name=name,
                provider=provider,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/secrets/integrations/create/{provider}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body, False, False, "json", models.DtoCreateIntegrationRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createOrUpdateIntegration",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.DtoSecretResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def list_linked_integrations(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoLinkedIntegrationsResponse:
        r"""List linked integrations

        Use when showing which integrations are connected (e.g. settings page). Returns providers that have valid linked credentials.

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request(
            method="GET",
            path="/secrets/integrations/linked",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listLinkedIntegrations",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.DtoLinkedIntegrationsResponse, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def list_linked_integrations_async(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoLinkedIntegrationsResponse:
        r"""List linked integrations

        Use when showing which integrations are connected (e.g. settings page). Returns providers that have valid linked credentials.

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request_async(
            method="GET",
            path="/secrets/integrations/linked",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listLinkedIntegrations",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.DtoLinkedIntegrationsResponse, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def delete_integration(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Delete an integration

        Use when disconnecting an integration (e.g. switching provider or removing OAuth). Deletes stored credentials.

        :param id: Integration ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeleteIntegrationRequest(
            id=id,
        )

        req = self._build_request(
            method="DELETE",
            path="/secrets/integrations/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deleteIntegration",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def delete_integration_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Delete an integration

        Use when disconnecting an integration (e.g. switching provider or removing OAuth). Deletes stored credentials.

        :param id: Integration ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeleteIntegrationRequest(
            id=id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/secrets/integrations/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deleteIntegration",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .types_filtercondition import TypesFilterCondition, TypesFilterConditionTypedDict
from .types_invoicestatus import TypesInvoiceStatus
from .types_invoicetype import TypesInvoiceType
from .types_paymentstatus import TypesPaymentStatus
from .types_sortcondition import TypesSortCondition, TypesSortConditionTypedDict
from .types_status import TypesStatus
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


TypesInvoiceFilterOrder = Literal[
    "asc",
    "desc",
]


class TypesInvoiceFilterTypedDict(TypedDict):
    amount_due_gt: NotRequired[float]
    r"""amount_due_gt filters invoices with a total amount due greater than the specified value
    Useful for finding invoices above a certain threshold or identifying high-value invoices
    """
    amount_remaining_gt: NotRequired[float]
    r"""amount_remaining_gt filters invoices with an outstanding balance greater than the specified value
    Useful for finding invoices that still have significant unpaid amounts
    """
    customer_id: NotRequired[str]
    r"""customer_id filters invoices for a specific customer using FlexPrice's internal customer ID
    This is the ID returned by FlexPrice when creating or retrieving customers
    """
    end_time: NotRequired[str]
    expand: NotRequired[str]
    external_customer_id: NotRequired[str]
    r"""external_customer_id filters invoices for a customer using your system's customer identifier
    This is the ID you provided when creating the customer in FlexPrice
    """
    filters: NotRequired[List[TypesFilterConditionTypedDict]]
    invoice_ids: NotRequired[List[str]]
    r"""invoice_ids restricts results to invoices with the specified IDs
    Use this to retrieve specific invoices when you know their exact identifiers
    """
    invoice_status: NotRequired[List[TypesInvoiceStatus]]
    r"""invoice_status filters by the current state of invoices in their lifecycle
    Multiple statuses can be specified to include invoices in any of the listed states
    """
    invoice_type: NotRequired[TypesInvoiceType]
    limit: NotRequired[int]
    offset: NotRequired[int]
    order: NotRequired[TypesInvoiceFilterOrder]
    payment_status: NotRequired[List[TypesPaymentStatus]]
    r"""payment_status filters by the payment state of invoices
    Multiple statuses can be specified to include invoices with any of the listed payment states
    """
    period_end_gte: NotRequired[str]
    r"""period_end_gte filters invoices with period_end >= value"""
    period_end_lte: NotRequired[str]
    r"""period_end_lte filters invoices with period_end <= value"""
    period_start_gte: NotRequired[str]
    r"""period_start_gte filters invoices with period_start >= value"""
    period_start_lte: NotRequired[str]
    r"""period_start_lte filters invoices with period_start <= value"""
    skip_line_items: NotRequired[bool]
    r"""SkipLineItems if true, will not include line items in the response"""
    sort: NotRequired[List[TypesSortConditionTypedDict]]
    start_time: NotRequired[str]
    status: NotRequired[TypesStatus]
    subscription_id: NotRequired[str]
    r"""subscription_id filters invoices generated for a specific subscription
    Only returns invoices that were created as part of the specified subscription's billing
    """


class TypesInvoiceFilter(BaseModel):
    amount_due_gt: Optional[float] = None
    r"""amount_due_gt filters invoices with a total amount due greater than the specified value
    Useful for finding invoices above a certain threshold or identifying high-value invoices
    """

    amount_remaining_gt: Optional[float] = None
    r"""amount_remaining_gt filters invoices with an outstanding balance greater than the specified value
    Useful for finding invoices that still have significant unpaid amounts
    """

    customer_id: Optional[str] = None
    r"""customer_id filters invoices for a specific customer using FlexPrice's internal customer ID
    This is the ID returned by FlexPrice when creating or retrieving customers
    """

    end_time: Optional[str] = None

    expand: Optional[str] = None

    external_customer_id: Optional[str] = None
    r"""external_customer_id filters invoices for a customer using your system's customer identifier
    This is the ID you provided when creating the customer in FlexPrice
    """

    filters: Optional[List[TypesFilterCondition]] = None

    invoice_ids: Optional[List[str]] = None
    r"""invoice_ids restricts results to invoices with the specified IDs
    Use this to retrieve specific invoices when you know their exact identifiers
    """

    invoice_status: Optional[List[TypesInvoiceStatus]] = None
    r"""invoice_status filters by the current state of invoices in their lifecycle
    Multiple statuses can be specified to include invoices in any of the listed states
    """

    invoice_type: Optional[TypesInvoiceType] = None

    limit: Optional[int] = None

    offset: Optional[int] = None

    order: Optional[TypesInvoiceFilterOrder] = None

    payment_status: Optional[List[TypesPaymentStatus]] = None
    r"""payment_status filters by the payment state of invoices
    Multiple statuses can be specified to include invoices with any of the listed payment states
    """

    period_end_gte: Optional[str] = None
    r"""period_end_gte filters invoices with period_end >= value"""

    period_end_lte: Optional[str] = None
    r"""period_end_lte filters invoices with period_end <= value"""

    period_start_gte: Optional[str] = None
    r"""period_start_gte filters invoices with period_start >= value"""

    period_start_lte: Optional[str] = None
    r"""period_start_lte filters invoices with period_start <= value"""

    skip_line_items: Optional[bool] = None
    r"""SkipLineItems if true, will not include line items in the response"""

    sort: Optional[List[TypesSortCondition]] = None

    start_time: Optional[str] = None

    status: Optional[TypesStatus] = None

    subscription_id: Optional[str] = None
    r"""subscription_id filters invoices generated for a specific subscription
    Only returns invoices that were created as part of the specified subscription's billing
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount_due_gt",
                "amount_remaining_gt",
                "customer_id",
                "end_time",
                "expand",
                "external_customer_id",
                "filters",
                "invoice_ids",
                "invoice_status",
                "invoice_type",
                "limit",
                "offset",
                "order",
                "payment_status",
                "period_end_gte",
                "period_end_lte",
                "period_start_gte",
                "period_start_lte",
                "skip_line_items",
                "sort",
                "start_time",
                "status",
                "subscription_id",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_usageresult import DtoUsageResult, DtoUsageResultTypedDict
from .types_aggregationtype import TypesAggregationType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoGetUsageResponseTypedDict(TypedDict):
    event_name: NotRequired[str]
    results: NotRequired[List[DtoUsageResultTypedDict]]
    type: NotRequired[TypesAggregationType]
    value: NotRequired[float]


class DtoGetUsageResponse(BaseModel):
    event_name: Optional[str] = None

    results: Optional[List[DtoUsageResult]] = None

    type: Optional[TypesAggregationType] = None

    value: Optional[float] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["event_name", "results", "type", "value"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_billingperiodinfo import DtoBillingPeriodInfo, DtoBillingPeriodInfoTypedDict
from .dto_featureusagesummary import (
    DtoFeatureUsageSummary,
    DtoFeatureUsageSummaryTypedDict,
)
from .types_paginationresponse import (
    TypesPaginationResponse,
    TypesPaginationResponseTypedDict,
)
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCustomerUsageSummaryResponseTypedDict(TypedDict):
    customer_id: NotRequired[str]
    features: NotRequired[List[DtoFeatureUsageSummaryTypedDict]]
    pagination: NotRequired[TypesPaginationResponseTypedDict]
    period: NotRequired[DtoBillingPeriodInfoTypedDict]


class DtoCustomerUsageSummaryResponse(BaseModel):
    customer_id: Optional[str] = None

    features: Optional[List[DtoFeatureUsageSummary]] = None

    pagination: Optional[TypesPaginationResponse] = None

    period: Optional[DtoBillingPeriodInfo] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["customer_id", "features", "pagination", "period"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import UnrecognizedStr
from typing import Literal, Union


TypesBillingModel = Union[
    Literal[
        "FLAT_FEE",
        "PACKAGE",
        "TIERED",
    ],
    UnrecognizedStr,
]
